import { Mapper } from '@mobicoop/ddd-library';
import { Injectable } from '@nestjs/common';
import { ConfigurationEntity } from './core/domain/configuration.entity';
import { ConfigurationResponseDto } from './interface/dtos/configuration.response.dto';
import { ConfigurationDomain } from './core/domain/configuration.types';
import {
  ConfigurationReadModel,
  ConfigurationWriteModel,
} from './infrastructure/configuration.repository';
import { v4 } from 'uuid';

@Injectable()
export class ConfigurationMapper
  implements
    Mapper<
      ConfigurationEntity,
      ConfigurationReadModel,
      ConfigurationWriteModel,
      ConfigurationResponseDto
    >
{
  toPersistence = (entity: ConfigurationEntity): ConfigurationWriteModel => {
    const copy = entity.getProps();
    const record: ConfigurationWriteModel = {
      key: `${copy.identifier.domain}:${copy.identifier.key}`,
      value: copy.value,
    };
    return record;
  };

  toDomain = (record: ConfigurationReadModel): ConfigurationEntity => {
    const entity = new ConfigurationEntity({
      id: v4(),
      createdAt: new Date(),
      updatedAt: new Date(),
      props: {
        identifier: {
          domain: record.key.split(':')[0] as ConfigurationDomain,
          key: record.key.split(':')[1],
        },
        value: record.value,
      },
    });
    return entity;
  };

  toResponse = (entity: ConfigurationEntity): ConfigurationResponseDto => {
    const props = entity.getProps();
    const response = new ConfigurationResponseDto(entity);
    response.domain = props.identifier.domain;
    response.key = props.identifier.key;
    response.value = props.value;
    return response;
  };
}
