import { CommandHandler, ICommandHandler } from '@nestjs/cqrs';
import { Inject } from '@nestjs/common';
import { SetConfigurationCommand } from './set-configuration.command';
import { CONFIGURATION_REPOSITORY } from '@modules/configuration/configuration.di-tokens';
import { ConfigurationDomain } from '@modules/configuration/core/domain/configuration.types';
import { ConfigurationRepositoryPort } from '../../ports/configuration.repository.port';

@CommandHandler(SetConfigurationCommand)
export class SetConfigurationService implements ICommandHandler {
  constructor(
    @Inject(CONFIGURATION_REPOSITORY)
    private readonly configurationRepository: ConfigurationRepositoryPort,
  ) {}

  async execute(command: SetConfigurationCommand): Promise<void> {
    await this.configurationRepository.set(
      {
        domain: command.domain as ConfigurationDomain,
        key: command.key,
      },
      command.value,
    );
  }
}
