import { NotFoundException } from '@mobicoop/ddd-library';
import { ConfigurationMapper } from '@modules/configuration/configuration.mapper';
import { ConfigurationEntity } from '@modules/configuration/core/domain/configuration.entity';
import { ConfigurationDomain } from '@modules/configuration/core/domain/configuration.types';
import { ConfigurationRepository } from '@modules/configuration/infrastructure/configuration.repository';
import { Test, TestingModule } from '@nestjs/testing';
import { getRedisToken } from '@songkeys/nestjs-redis';

const mockRedis = {
  get: jest
    .fn()
    .mockImplementationOnce(() => '1')
    .mockImplementation(() => null),
  set: jest.fn().mockImplementation(),
};

const mockConfigurationMapper = {
  toDomain: jest.fn().mockImplementation(
    () =>
      new ConfigurationEntity({
        id: ' 001199d4-7187-4e83-a044-12159cba2e33',
        props: {
          identifier: {
            domain: ConfigurationDomain.CARPOOL,
            key: 'seatsProposed',
          },
          value: '1',
        },
        createdAt: new Date('2023-10-23'),
        updatedAt: new Date('2023-10-23'),
      }),
  ),
};

describe('Configuration Repository', () => {
  let configurationRepository: ConfigurationRepository;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: getRedisToken('default'),
          useValue: mockRedis,
        },
        {
          provide: ConfigurationMapper,
          useValue: mockConfigurationMapper,
        },
        ConfigurationRepository,
      ],
    }).compile();

    configurationRepository = module.get<ConfigurationRepository>(
      ConfigurationRepository,
    );
  });

  it('should be defined', () => {
    expect(configurationRepository).toBeDefined();
  });

  describe('interact', () => {
    it('should get a value', async () => {
      expect(
        (
          await configurationRepository.get({
            domain: ConfigurationDomain.CARPOOL,
            key: 'seatsProposed',
          })
        ).getProps().value,
      ).toBe('1');
    });
    it('should throw if configuration is not found', async () => {
      await expect(
        configurationRepository.get({
          domain: ConfigurationDomain.CARPOOL,
          key: 'seatsProposed',
        }),
      ).rejects.toBeInstanceOf(NotFoundException);
    });
    it('should set a value', async () => {
      expect(
        await configurationRepository.set(
          {
            domain: ConfigurationDomain.CARPOOL,
            key: 'seatsProposed',
          },
          '3',
        ),
      ).toBeUndefined();
    });
  });
});
